#!/bin/sh
curpath=`pwd`
scriptpath=${0%/*}
arguments=$*
tmpfile=tmp
shellopt=-x
ADBPATH=bin/adb
TARGETPKG=com.x0.strai.secondfrep
TARGETCLS=com.x0.strai.secondfrep.StrServ

if [ $0 = $scriptpath ] ; then
  scriptpath=.
fi

funcerros() {
  echo "FRep2はAndroid 7以降でのみ動作します."
  echo "Android 6以前ではFRepを使用してください."
  echo "[FRep] http://strai.x0.com/frep-ja/"
  cd $curpath
  exit 1
}

funcerrnotgt() {
  echo "FRep2がインストールされていないか、インストール後の設定対象にアクセスできません."
  echo "(サポート用)以下のコマンドを実行した結果をサポートまでご送付ください."
  echo "$scriptpath/$ADBPATH shell \"pm list packages -f | grep =$TARGETPKG\""
  cd $curpath
  exit 1
}

funcerr() {
  echo "FRep2精密モードの設定に失敗しました."
  echo "- Androidの設定より開発者オプションのUSBデバッグをONにしてください."
  echo "- その他の問題はサポートサイトを参照してください."
  echo "http://strai.x0.com/frep2-ja/"
  cd $curpath
  exit 1
}

funcoffline() {
  echo "Android機器に表示されているUSBデバッグを許可してからEnterキーを押してください.(中止はCtrl+C)"
  read tmpinput
}

funcnumeric() {
  numcand=$1
  expr "$numcand" + 1 >/dev/null 2>&1
  # -lt 2 Numeric
  if [ $? -ge 2 ] ; then
    numcand=0
  fi
  echo $numcand
# return $numcand
}

funcdevice() {
  echo "複数のAndroid機器が接続されています."
  devnum=0
  $ADBPATH devices -l >$tmpfile
  # while~do~done with redirecting file because of variable lost.
  while read line ; do
    if [ -n "$line" ] ; then
      serial="`echo $line | awk '{print $1}'`"
#      serial=${line%%device*}
#      serial=${serial%% *  }
      product=${line#*product:}
      product=${product%transport_id*}
#      tid=${line#*transport_id}
      if [ $devnum -gt 0 ] ; then
        echo " ($devnum) $serial:$product"
#        echo " ($devnum) $serial:$product tid$tid"
      fi
      devnum=`expr $devnum + 1`
    fi
  done <$tmpfile

  echo "設定する機器の番号を入力してください.(0でキャンセル)"
  read selnum
  seldevice=
  devnum=0
  selnum=`funcnumeric $selnum`

  if [ $selnum -eq 0 ] ; then
    echo "設定を中止しました."
    exit 0
  else
    # while~do~done with redirecting file because of variable lost.
    while read line ; do
    if [ -n "$line" ] ; then
      if [ $devnum -eq $selnum ] ; then
        seldevice="`echo $line | awk '{print $1}'`"
        break
      fi
      devnum=`expr $devnum + 1`
    fi
    done <$tmpfile
  fi
  rm $tmpfile

  if [ -n $seldevice ] ; then
    arguments="-s $seldevice"
    echo "($selnum) $seldevice を選択しました."
  else
    echo "設定する機器1台のみ接続した状態で再度実行するか、あるいは設定する機器1台を"
    echo "オプション -s (上記シリアルの1つ) で指定してください."
    echo "例) frep2.sh -s 00000000"
    exit 1
  fi
}

# get pid in ps -ef line.
getsecond() {
  case $# in
    0)
    return 0
    ;;
    1)
    return 0
    ;;
  esac
  echo $2

}
# Check process and kill if already running. (Android 8~, since ~7.1 does not have ps -ef)
killserv() {
  $ADBPATH $arguments shell "ps -ef | grep $TARGETCLS" >$tmpfile
  # while~do~done with redirecting file because of variable lost.
  while read line ; do
    case $line in
      *app_process*$TARGETCLS)
        PID=`getsecond $line`
        # echo $PID in $line
        case $PID in
          "");;
          0);;
          *)
          $ADBPATH $arguments shell kill $PID
          echo PID $PID strserv killed.
          ;;
        esac
        ;;
      *);;
    esac
  done <$tmpfile
  rm $tmpfile
}

cd $scriptpath
chmod +x $ADBPATH
$ADBPATH devices > /dev/null 2>&1

echo "[FRep2 精密モード 設定ツール]"

# check Android version and USB debug connection
passed=0
while [ $passed -eq 0 ] ; do
  versionsdk=`$ADBPATH $arguments shell getprop ro.build.version.sdk 2>&1`
  case $? in
    # passed (no error)
    0)
    passed=1
    ;;
    # error
    *)
    case $versionsdk in
      # skip sdk check when getprop denied or not found.
      *enied)
      passed=1
      ;;
      *getprop*found)
      passed=1
      ;;
      # error: more than one device/emulator (continue)
      *emulator)
      funcdevice
      ;;
      # error: device offline (continue)
      *offline)
      funcoffline
      ;;
      # error: device unauthorized (continue)
      *unauthorized*)
      funcoffline
      ;;
      # other error
      *)
      echo $versionsdk
      funcerr
      ;;
    esac
    ;;
  esac
done

# not work on Android ~6
if [ $versionsdk -le 23 ] ; then
  funcerros
fi

# adjust option for Android 10 or later
if [ $versionsdk -ge 29 ] ; then
  shellopt=
fi

# force stop running process on Android 8~
if [ $versionsdk -ge 26 ] ; then
  killserv
fi

PKGPATH=`$ADBPATH $arguments shell pm path $TARGETPKG`
PKGPATHLEN=${#PKGPATH}
PKGPATH=`echo $PKGPATH | cut -c 9-$PKGPATHLEN`
ADBCMD="export CLASSPATH=$PKGPATH && app_process /system/bin $TARGETCLS"

if [ -n "$ADBCMD" ]; then
  echo "StrServ Startedと表示されたらCtrl+Cを押してスクリプトを抜けてください."
  echo ""
#  echo $ADBPATH $arguments shell $shellopt "$ADBCMD"
  echo "$ADBCMD &" | $ADBPATH $arguments shell $shellopt
else
  funcerrnotgt
fi

cd $curpath
exit 0

